#ifndef DPX_DEVICE_TOUCH_H
#define DPX_DEVICE_TOUCH_H

#include "DeviceAPI.h"

/*
    Touch states enum
*/
enum DPX_DEVICE_TOUCH_STATE
{
    TOUCH_STATE_DOWN,
    TOUCH_STATE_MOVE,
    TOUCH_STATE_UP,
    TOUCH_STATE_INVALID
};

/*
  Structure that defines a touch
*/
typedef struct _DPX_DEVICE_TOUCH
{
    int Id;
    int X;
    int Y;
    int Pressure;
    enum DPX_DEVICE_TOUCH_STATE State;
} DPX_DEVICE_TOUCH, *PDPX_DEVICE_TOUCH;

/*
    Name: DPXDeviceBeginTouchIterate
    Description: Begins touch iteration. You need to lock the device to ensure that touch data is coherent.
    Parameters: DPX_DEVICE pointer
    Return value: None
*/
void DPXDeviceBeginTouchIterate(PDPX_DEVICE device);

/*
    Name: DPXDeviceNextTouch
    Description: Get the next available touch. Returns true if more touches are available, otherwise returns false.
    Parameters: DPX_CONTEXT pointer, DPX_DEVICE_TOUCH pointer
    Return value: bool
*/
int DPXDeviceNextTouch(PDPX_DEVICE device, PDPX_DEVICE_TOUCH);

/*
    Name: DPXDeviceEndTouchIterate
    Description: Ends touch iteration. You need to unlock the device after this call.
    Parameters: DPX_DEVICE pointer
    Return value: None
*/
void DPXDeviceEndTouchIterate(PDPX_DEVICE device);

#endif // DPX_DEVICE_TOUCH_H
