#include <stdio.h>
#include <stdlib.h>

#include "TouchAPI.h"
#include "DeviceAPI.h"

int main(int argc, char *argv[])
{
    (void) argc;
    (void) argv;

    PDPX_CONTEXT context = DPXContextCreate();

    if (!context)
    {
        printf("Couldn't create context\n");
        fflush(stdout);
        return EXIT_FAILURE;
    }

    printf("Retrieving device count\n");
    fflush(stdout);

    DPXContextUpdateDevices(context);

    int count = DPXContextGetDeviceCount(context);

    if (count==0)
    {
        printf("No devices available\n");
        fflush(stdout);
        return EXIT_FAILURE;
    }

    printf("Found %d devices\n", count);
    fflush(stdout);

    int i;

    PDPX_DEVICE device;

    for(i=0;i<count;i++)
    {
        device = DPXContextGetDevice(context, i);

        if (!device)
        {
            printf("Couldn't open device\n");
            fflush(stdout);
            continue;
        }

        DPXDeviceOpen(device);

        printf("[%d] %s\n",i+1, DPXDeviceGetSerial(device));
        fflush(stdout);

        DPXDeviceClose(device);
    }

    printf("Iterating over first device touches\n");
    fflush(stdout);

    device = DPXContextGetDevice(context, 0);

    if (!device)
    {
        printf("Couldn't open device\n");
        fflush(stdout);
    }

    DPXDeviceOpen(device);

    #if 1
    while(1)
    {
        DPX_DEVICE_TOUCH touch;

        DPXDeviceBeginTouchIterate(device);

        while(DPXDeviceNextTouch(device, &touch))
        {
            printf("Id:%d X:%d Y:%d Pressure:%d\n",touch.Id,touch.X,touch.Y,touch.Pressure);
            fflush(stdout);
        }

        DPXDeviceEndTouchIterate(device);
    }
    #else
    unsigned char frame[4096];

    while(1)
    {
        DPXDeviceLock(device);
        DPXDeviceCopyFrame(device, frame, 4096);
        DPXDeviceUnlock(device);

        for (int i=0;i<4096;i++)
        {
            printf("%03d",frame[i]);
            fflush(stdout);

            if (i%64==0)
                printf("\n");
        }

        printf("\n\n");
    }
    #endif

    DPXContextDestroy(context);

    return 0;
}
