#include "dpxmtt.h"
#include <linux/version.h>

static unsigned long tInitial;
static unsigned long tCurrent;
static unsigned long elapsed;

static void dpx_data_work(struct work_struct *work);

static int dpx_alloc(struct usbtouch_usb *usbtouch)
{
    struct dpx_priv *priv;
    int ret = -ENOMEM;

    //printk(KERN_INFO "DPX Alloc");

    usbtouch->priv = kmalloc(sizeof(struct dpx_priv), GFP_KERNEL);
    if (!usbtouch->priv)
        goto out_buf;

    priv = usbtouch->priv;

    priv->context = kmalloc(sizeof(DPX_DEVICE), GFP_KERNEL);

    if (!priv->context)
        goto err_priv;

    return 0;

err_priv:
    kfree(priv);
out_buf:
    return ret;
}

static int dpx_init(struct usbtouch_usb *usbtouch)
{
    struct usb_device *dev = interface_to_usbdev(usbtouch->interface);
    //struct usb_host_interface *interface = usbtouch->interface->cur_altsetting;
    struct dpx_priv *priv = usbtouch->priv;
    char *p,*p1;
    char *buf;
    int i;

    printk(KERN_INFO "Initializing DISPLAX SKIN MULTITOUCH kernel module");
    DPXDeviceInit(priv->context);

    buf = 0;
    buf = kmalloc(sizeof(char)*STRING_LENGTH, GFP_KERNEL);

    if (!buf)
    {
        printk(KERN_INFO "failed to alloc");
    }
    else
    {
        priv->context->Pid = dev->descriptor.idProduct;

        p1 = buf;

        usb_string(dev, dev->descriptor.iManufacturer,buf, STRING_LENGTH);
        printk(KERN_INFO "Manufacturer name: %s",buf);

        p = priv->context->State.Acquisition.Manufacturer;

        for (i=0;i<STRING_LENGTH;i++)
        {
            *p++ = *p1++;
        }

        p1 = buf;

        usb_string(dev, dev->descriptor.iProduct,buf, STRING_LENGTH);
        printk(KERN_INFO "Product name: %s",buf);

        p = priv->context->State.Acquisition.Product;

        for (i=0;i<STRING_LENGTH;i++)
        {
            *p++ = *p1++;
        }

        p1 = buf;

        usb_string(dev, dev->descriptor.iSerialNumber,buf, STRING_LENGTH);
        printk(KERN_INFO "Serial number: %s",buf);

        p = priv->context->State.Acquisition.Serial;

        for (i=0;i<STRING_LENGTH;i++)
        {
            *p++ = *p1++;
        }
    }

    kfree(buf);

    tInitial = jiffies;

    return 0;
}

static void dpx_exit(struct usbtouch_usb *usbtouch)
{
    struct dpx_priv *priv = usbtouch->priv;

    flush_scheduled_work();

    kfree(priv->context);
    kfree(priv);
}

static int dpx_read_data(struct usbtouch_usb *usbtouch, unsigned char *pkt)
{
    int r=0;
    struct dpx_priv *priv = usbtouch->priv;

    DPXDeviceSetConnectionState(priv->context, DPX_STATE_CONNECTED);

    if (priv->context->Pid==1)
    {
        r=DPXDeviceFillFrameV1(priv->context, pkt, 2048);
    }
    else
    {
        r=DPXDeviceFillFrameV3(priv->context, pkt, 2048);
    }

    if (r)
    {
        schedule_work(&usbtouch->data_work);
    }

    return 1;
}

static struct usbtouch_device_info usbtouch_dev_info[] = {

    [DEVTYPE_DPX] = {
        .min_xc		= 0x0,
        .max_xc		= 0x189C,
        .min_yc		= 0x0,
        .max_yc		= 0x189C,
        .rept_size	= 2048,
        .irq_always	= true,
        .read_data	= dpx_read_data,
        .alloc		= dpx_alloc,
        .init		= dpx_init,
        .exit		= dpx_exit,
    },
};

void dpx_set_gain(struct usbtouch_usb *usbtouch)
{
    PDPX_DEVICE device;
    unsigned char requestPacket[8];
    int transferedBytes;

    struct dpx_priv *priv = usbtouch->priv;
    struct usb_device *usbdev = interface_to_usbdev(usbtouch->interface);

    device = priv->context;

    device->State.Acquisition.Gain = priv->context->State.AutoGainValue;
    DPXDeviceFillRequestPacket(device, requestPacket, DPX_REQ_SET_PARAMETER);
    usb_bulk_msg(usbdev, usb_sndbulkpipe(usbdev, (int)0x02), &requestPacket, 8, &transferedBytes, 1000);
}

void dpx_calibration(struct usbtouch_usb *usbtouch, unsigned char type)
{
    PDPX_DEVICE device;
    unsigned char requestPacket[8];
    int transferedBytes;

    struct dpx_priv *priv = usbtouch->priv;
    struct usb_device *usbdev = interface_to_usbdev(usbtouch->interface);

    device = priv->context;

    DPXDeviceFillRequestPacket(device, requestPacket, type);
    usb_bulk_msg(usbdev, usb_sndbulkpipe(usbdev, (int)0x02), &requestPacket, 8, &transferedBytes, 1000);
}

static void dpx_data_work(struct work_struct *work)
{
    int x=0,y=0,touchCount=0,currentTouch=0,down=0;
    PDPX_TOUCH Touch;

    struct usbtouch_usb *usbtouch = container_of(work, struct usbtouch_usb, data_work);
    struct dpx_priv *priv = usbtouch->priv;
    struct input_dev *input = usbtouch->input;

    if (!priv->context->State.Acquisition.Status)
        return;

    DPXDeviceProcessFrame(priv->context);

    DPXDevicePerformFalseTouchesCheck(usbtouch);
    DPXDevicePerformPartialCalibrationCheck(usbtouch);

    if (!priv->context->State.Acquisition.Status)
        return;

    if (!priv->context->State.Enabled)
        return;

    if(priv->context->Data.Tracking.TouchCount==0)
        return;
            
#if 1
    mutex_lock(&input->mutex);

    for (currentTouch=0;currentTouch<DPX_TOUCH_MAX_COUNT && touchCount<priv->context->State.Acquisition.TouchPoints;++currentTouch)
    {
        Touch = priv->context->State.Touches + currentTouch;

        x = Touch->CalibratedPoint.Position.X;
        y = Touch->CalibratedPoint.Position.Y;

        // touch down
        if(Touch->CurrentState==DPX_TOUCH_STATE_ACTIVE && Touch->ReportedState==DPX_TOUCH_STATE_INACTIVE)
        {
//            printk(KERN_INFO "touch id %d down: x: %d y: %d\n",Touch->Id,x,y);
            Touch->ReportedState = DPX_TOUCH_STATE_ACTIVE;
            down = 1;
            touchCount++;
        }
        // touch move
        else if (Touch->CurrentState==DPX_TOUCH_STATE_ACTIVE && Touch->ReportedState==DPX_TOUCH_STATE_ACTIVE)
        {
//            printk(KERN_INFO "touch id %d move: x: %d y: %d\n",Touch->Id,x,y);
            Touch->ReportedState = DPX_TOUCH_STATE_ACTIVE;
            down = 1;
    //        printk(KERN_INFO "%d %d\n", x, y);
            touchCount++;
        }
        // touch up
        else if(Touch->CurrentState==DPX_TOUCH_STATE_INACTIVE && Touch->ReportedState==DPX_TOUCH_STATE_ACTIVE)
        {
//            printk(KERN_INFO "touch id %d up: x: %d y: %d\n",Touch->Id,x,y);
            Touch->ReportedState = DPX_TOUCH_STATE_INACTIVE;
            down = 0;
            touchCount++;
        }
        else
        {
            continue;
        }

        input_mt_slot(usbtouch->input, currentTouch);

        if (down)
        {
            input_report_abs(usbtouch->input, ABS_MT_POSITION_X, x);
            input_report_abs(usbtouch->input, ABS_MT_POSITION_Y, y);
            //input_report_abs(usbtouch->input, ABS_MT_TOOL_X, x);
            //input_report_abs(usbtouch->input, ABS_MT_TOOL_Y, y);
            input_report_abs(usbtouch->input, ABS_MT_TOUCH_MAJOR, 1);
            //input_report_abs(usbtouch->input, ABS_MT_TOUCH_MINOR, 1);
        }

        input_mt_report_slot_state(usbtouch->input, MT_TOOL_FINGER, down);
    }

    input_sync(usbtouch->input);

    mutex_unlock(&input->mutex);
#else
    mutex_lock(&input->mutex);

    for (currentTouch=0;currentTouch<DPX_TOUCH_MAX_COUNT && touchCount<priv->context->State.Acquisition.TouchPoints;++currentTouch)
    {
        Touch = priv->context->State.Touches + currentTouch;

        x = Touch->CalibratedPoint.Position.X;
        y = Touch->CalibratedPoint.Position.Y;


        input_mt_slot(usbtouch->input, currentTouch);

        // touch down
        if(Touch->CurrentState==DPX_TOUCH_STATE_ACTIVE && Touch->ReportedState==DPX_TOUCH_STATE_INACTIVE)
        {
            printk(KERN_INFO "touch id %d down: x: %d y: %d\n",Touch->Id,x,y);
            Touch->ReportedState = DPX_TOUCH_STATE_ACTIVE;

            input_mt_report_slot_state(usbtouch->input, MT_TOOL_FINGER, 1);

            input_report_abs(usbtouch->input, ABS_MT_POSITION_X, x);
            input_report_abs(usbtouch->input, ABS_MT_POSITION_Y, y);

            touchCount++;
        }
        // touch move
        else if (Touch->CurrentState==DPX_TOUCH_STATE_ACTIVE && Touch->ReportedState==DPX_TOUCH_STATE_ACTIVE)
        {
            printk(KERN_INFO "touch id: %d move: x: %d y: %d\n",Touch->Id,x,y);
            Touch->ReportedState = DPX_TOUCH_STATE_ACTIVE;

            input_mt_report_slot_state(usbtouch->input, MT_TOOL_FINGER, 1);

            input_report_abs(usbtouch->input, ABS_MT_POSITION_X, x);
            input_report_abs(usbtouch->input, ABS_MT_POSITION_Y, y);

            touchCount++;
        }
        // touch up
        else
        {
            printk(KERN_INFO "touch id: %d up: x: %d y: %d\n",Touch->Id,x,y);
            Touch->ReportedState = DPX_TOUCH_STATE_INACTIVE;

            input_mt_report_slot_state(usbtouch->input, MT_TOOL_FINGER, 0);

            touchCount++;
        }
    }

    if (touchCount>0)
    {
        input_mt_report_pointer_emulation(usbtouch->input, true);
        input_sync(usbtouch->input);
    }

    mutex_unlock(&input->mutex);
#endif
}

static void usbtouch_process_pkt(struct usbtouch_usb *usbtouch, unsigned char *pkt, int len)
{

    struct usbtouch_device_info *type = usbtouch->type;
    struct dpx_priv *priv = usbtouch->priv;

    if (!type->read_data(usbtouch, pkt))
        return;

    tCurrent = jiffies;
    elapsed = tCurrent-tInitial;
    elapsed = elapsed * 1000 / HZ;

    if (elapsed>1000)
    {
        priv->context->State.Acquisition.FrameRate = priv->context->Data.Acquisition.Frames;
        priv->context->Data.Acquisition.Frames = 0;
        priv->context->State.Tracking.FalseTouches = 0;

        tInitial = tCurrent;
    }
}

static void usbtouch_irq(struct urb *urb)
{
    struct usbtouch_usb *usbtouch = urb->context;
    int retval;

    switch (urb->status) {
    case 0:
        /* success */
        break;
    case -ETIME:
        /* this urb is timing out */
        dbg("%s - urb timed out - was the device unplugged?",
            __func__);
        return;
    case -ECONNRESET:
    case -ENOENT:
    case -ESHUTDOWN:
    case -EPIPE:
        /* this urb is terminated, clean up */
        dbg("%s - urb shutting down with status: %d",
            __func__, urb->status);
        return;
    default:
        dbg("%s - nonzero urb status received: %d",
            __func__, urb->status);
        goto exit;
    }

    usbtouch->type->process_pkt(usbtouch, usbtouch->data, urb->actual_length);

exit:
    usb_mark_last_busy(interface_to_usbdev(usbtouch->interface));
    retval = usb_submit_urb(urb, GFP_ATOMIC);
    if (retval)
        err("%s - usb_submit_urb failed with result: %d",
            __func__, retval);
}

static int usbtouch_open(struct input_dev *input)
{
    struct usbtouch_usb *usbtouch = input_get_drvdata(input);
    int r;

    usbtouch->irq->dev = interface_to_usbdev(usbtouch->interface);

    r = usb_autopm_get_interface(usbtouch->interface) ? -EIO : 0;
    if (r < 0)
        goto out;

    if (!usbtouch->type->irq_always) {
        if (usb_submit_urb(usbtouch->irq, GFP_KERNEL)) {
            r = -EIO;
            goto out_put;
        }
    }

    usbtouch->interface->needs_remote_wakeup = 1;
out_put:
    usb_autopm_put_interface(usbtouch->interface);
out:
    return r;
}

static void usbtouch_close(struct input_dev *input)
{
    struct usbtouch_usb *usbtouch = input_get_drvdata(input);
    int r;

    if (!usbtouch->type->irq_always)
        usb_kill_urb(usbtouch->irq);
    r = usb_autopm_get_interface(usbtouch->interface);
    usbtouch->interface->needs_remote_wakeup = 0;
    if (!r)
        usb_autopm_put_interface(usbtouch->interface);
}

static int usbtouch_suspend(struct usb_interface *intf, pm_message_t message)
{
    struct usbtouch_usb *usbtouch = usb_get_intfdata(intf);

    usb_kill_urb(usbtouch->irq);

    return 0;
}

static int usbtouch_resume(struct usb_interface *intf)
{
    struct usbtouch_usb *usbtouch = usb_get_intfdata(intf);
    struct input_dev *input = usbtouch->input;
    int result = 0;

    mutex_lock(&input->mutex);
    if (input->users || usbtouch->type->irq_always)
        result = usb_submit_urb(usbtouch->irq, GFP_NOIO);
    mutex_unlock(&input->mutex);

    return result;
}

static int usbtouch_reset_resume(struct usb_interface *intf)
{
    struct usbtouch_usb *usbtouch = usb_get_intfdata(intf);
    struct input_dev *input = usbtouch->input;
    int err = 0;

    /* reinit the device */
    if (usbtouch->type->init) {
        err = usbtouch->type->init(usbtouch);
        if (err) {
            dbg("%s - type->init() failed, err: %d",
                __func__, err);
            return err;
        }
    }

    /* restart IO if needed */
    mutex_lock(&input->mutex);
    if (input->users)
        err = usb_submit_urb(usbtouch->irq, GFP_NOIO);
    mutex_unlock(&input->mutex);

    return err;
}

static void usbtouch_free_buffers(struct usb_device *udev, struct usbtouch_usb *usbtouch)
{
    usb_free_coherent(udev, usbtouch->type->rept_size, usbtouch->data, usbtouch->data_dma);
    kfree(usbtouch->buffer);
}

static struct usb_endpoint_descriptor * usbtouch_get_input_endpoint(struct usb_host_interface *interface)
{
    int i;

    for (i = 0; i < interface->desc.bNumEndpoints; i++)
        if (usb_endpoint_dir_in(&interface->endpoint[i].desc))
            return &interface->endpoint[i].desc;

    return NULL;
}

static ssize_t read_state(struct file *filp, struct kobject *kobj, struct bin_attribute *attr, char *buf, loff_t off, size_t count)
{
    struct device *dev = container_of(kobj, struct device, kobj);
    struct usbtouch_usb *usbtouch = dev_get_drvdata(dev);
    struct dpx_priv *priv = usbtouch->priv;
    void *p;

    p=&priv->context->State;
    memcpy(buf,p+off,count);

    return count;
}

static ssize_t write_state(struct file *filp, struct kobject *kobj, struct bin_attribute *attr, char *buf, loff_t off, size_t count)
{
    struct device *dev = container_of(kobj, struct device, kobj);
    struct usbtouch_usb *usbtouch = dev_get_drvdata(dev);
    struct dpx_priv *priv = usbtouch->priv;

    void *p=&priv->context->State;
    memcpy(p+off,buf,count);

    DPXProcessingInitialize(priv->context,priv->context->State.Processing.NoiseReduction);

    return count;
}

static struct bin_attribute dev_state_bin_attr_descriptors = {
    .attr = {.name = "state", .mode = 0777},
    .read = read_state,
    .write = write_state,
    .size = sizeof(DPX_DEVICE_STATE)
};

static ssize_t write_bulk(struct file *filp, struct kobject *kobj, struct bin_attribute *attr, char *buf, loff_t off, size_t count)
{
    int transferedBytes=0;
    unsigned char requestPacket[8];
    struct device *dev = container_of(kobj, struct device, kobj);
    struct usbtouch_usb *usbtouch = dev_get_drvdata(dev);
    struct usb_device *usbdev = interface_to_usbdev(usbtouch->interface);

    if (count>0)
    {
        void *p=&requestPacket;

        memcpy(p+off,buf,count);

        usb_bulk_msg(usbdev, usb_sndbulkpipe(usbdev, (int)0x02), &requestPacket, 8, &transferedBytes, 1000);
    }

    return count;
}

static struct bin_attribute dev_bulk_bin_attr_descriptors = {
        .attr = {.name = "bulk", .mode = 0777},
        .write = write_bulk,
        .size = 8,
};

static int usbtouch_probe(struct usb_interface *intf, const struct usb_device_id *id)
{
    struct usbtouch_usb *usbtouch=0;
    struct input_dev *input_dev;
    struct usb_endpoint_descriptor *endpoint;
    struct usb_device *udev = interface_to_usbdev(intf);
    struct usbtouch_device_info *type;
    struct dpx_priv *priv = 0;
    int error = -ENOMEM;

    printk(KERN_INFO "Starting device probe");

    /* some devices are ignored */
    if (id->driver_info == DEVTYPE_IGNORE)
        return -ENODEV;

    endpoint = usbtouch_get_input_endpoint(intf->cur_altsetting);
    if (!endpoint)
        return -ENXIO;

    usbtouch = kzalloc(sizeof(struct usbtouch_usb), GFP_KERNEL);
    input_dev = input_allocate_device();
    if (!usbtouch || !input_dev)
        goto out_free;

    type = &usbtouch_dev_info[id->driver_info];
    usbtouch->type = type;
    if (!type->process_pkt)
        type->process_pkt = usbtouch_process_pkt;

    usbtouch->data = usb_alloc_coherent(udev, type->rept_size, GFP_KERNEL, &usbtouch->data_dma);
    if (!usbtouch->data)
        goto out_free;

    if (type->get_pkt_len) {
        usbtouch->buffer = kmalloc(type->rept_size, GFP_KERNEL);
        if (!usbtouch->buffer)
            goto out_free_buffers;
    }

    usbtouch->irq = usb_alloc_urb(0, GFP_KERNEL);

    if (!usbtouch->irq) {
        dbg("%s - usb_alloc_urb failed: usbtouch->irq", __func__);
        goto out_free_buffers;
    }

    usbtouch->interface = intf;
    usbtouch->input = input_dev;

    priv = usbtouch->priv;

    INIT_WORK(&usbtouch->data_work, dpx_data_work);

    if (udev->manufacturer)
        strlcpy(usbtouch->name, udev->manufacturer, sizeof(usbtouch->name));

    if (udev->product) {
        if (udev->manufacturer)
            strlcat(usbtouch->name, " ", sizeof(usbtouch->name));
        strlcat(usbtouch->name, udev->product, sizeof(usbtouch->name));
    }

    if (!strlen(usbtouch->name))
        snprintf(usbtouch->name, sizeof(usbtouch->name),
                 "USB Touchscreen %04x:%04x",
                 le16_to_cpu(udev->descriptor.idVendor),
                 le16_to_cpu(udev->descriptor.idProduct));

    printk(KERN_INFO "%d:%d", udev->descriptor.idVendor, udev->descriptor.idProduct);

    usb_make_path(udev, usbtouch->phys, sizeof(usbtouch->phys));
    strlcat(usbtouch->phys, "/input0", sizeof(usbtouch->phys));

    input_dev->name = usbtouch->name;
    input_dev->phys = usbtouch->phys;
    usb_to_input_id(udev, &input_dev->id);
    input_dev->dev.parent = &intf->dev;

    input_set_drvdata(input_dev, usbtouch);

    input_dev->open = usbtouch_open;
    input_dev->close = usbtouch_close;

    input_dev->evbit[0] = BIT_MASK(EV_KEY) | BIT_MASK(EV_ABS);
    input_dev->keybit[BIT_WORD(BTN_TOUCH)] = BIT_MASK(BTN_TOUCH);

    /* For single touch */
    input_set_abs_params(input_dev, ABS_X, 0, 6300, 0, 0);
    input_set_abs_params(input_dev, ABS_Y, 0, 6300, 0, 0);

    #if LINUX_VERSION_CODE < KERNEL_VERSION(3,7,0)
    input_mt_init_slots(input_dev, DPX_TOUCH_MAX_COUNT);
    #else
    input_mt_init_slots(input_dev, DPX_TOUCH_MAX_COUNT, INPUT_MT_DIRECT);
    #endif

    //input_set_abs_params(input_dev, ABS_MT_TOUCH_MAJOR, 0, 0xff, 0, 0);
    input_set_abs_params(input_dev, ABS_MT_POSITION_X, 0, 6300, 0, 0);
    input_set_abs_params(input_dev, ABS_MT_POSITION_Y, 0, 6300, 0, 0);

    //if (type->max_press)
//        input_set_abs_params(input_dev, ABS_PRESSURE, type->min_press, type->max_press, 0, 0);

    usb_fill_bulk_urb(usbtouch->irq, udev, usb_rcvbulkpipe(udev, endpoint->bEndpointAddress), usbtouch->data, type->rept_size, usbtouch_irq, usbtouch);

    usbtouch->irq->dev = udev;
    usbtouch->irq->transfer_dma = usbtouch->data_dma;
    usbtouch->irq->transfer_flags |= URB_NO_TRANSFER_DMA_MAP;

    /* device specific allocations */
    if (type->alloc) {
        error = type->alloc(usbtouch);
        if (error) {
            dbg("%s - type->alloc() failed, err: %d", __func__, error);
            goto out_free_urb;
        }
    }

    /* device specific initialisation*/
    if (type->init) {
        error = type->init(usbtouch);
        if (error) {
            dbg("%s - type->init() failed, err: %d", __func__, error);
            goto out_do_exit;
        }
    }

    error = input_register_device(usbtouch->input);

    if (error) {
        dbg("%s - input_register_device failed, err: %d", __func__, error);
        goto out_do_exit;
    }

    usb_set_intfdata(intf, usbtouch);

    if (usbtouch->type->irq_always)
    {
        /* this can't fail */
        usb_autopm_get_interface(intf);
        error = usb_submit_urb(usbtouch->irq, GFP_KERNEL);
        if (error) {
            usb_autopm_put_interface(intf);
            err("%s - usb_submit_urb failed with result: %d", __func__, error);
            goto out_unregister_input;
        }
    }

    if (sysfs_create_bin_file(&intf->dev.kobj, &dev_state_bin_attr_descriptors))
        goto out_unregister_input;

    if (sysfs_create_bin_file(&intf->dev.kobj, &dev_bulk_bin_attr_descriptors))
        goto out_unregister_input;

    return 0;

out_unregister_input:
    input_unregister_device(input_dev);
    input_dev = NULL;
out_do_exit:
    if (type->exit)
        type->exit(usbtouch);
out_free_urb:
    usb_free_urb(usbtouch->irq);
out_free_buffers:
    usbtouch_free_buffers(udev, usbtouch);
out_free:
    input_free_device(input_dev);
    kfree(usbtouch);
    return error;
}

static void usbtouch_disconnect(struct usb_interface *intf)
{
    struct usbtouch_usb *usbtouch = usb_get_intfdata(intf);

    dbg("%s - called", __func__);

    if (!usbtouch)
        return;

    dbg("%s - usbtouch is initialized, cleaning up", __func__);
    sysfs_remove_bin_file(&intf->dev.kobj, &dev_state_bin_attr_descriptors);
    sysfs_remove_bin_file(&intf->dev.kobj, &dev_bulk_bin_attr_descriptors);
    usb_set_intfdata(intf, NULL);
    /* this will stop IO via close */
    input_unregister_device(usbtouch->input);
    usb_free_urb(usbtouch->irq);

    if (usbtouch->type->exit)
        usbtouch->type->exit(usbtouch);

    usbtouch_free_buffers(interface_to_usbdev(intf), usbtouch);
    kfree(usbtouch);
}

MODULE_DEVICE_TABLE(usb, usbtouch_devices);

static struct usb_driver usbtouch_driver = {
    .name = "dpxmtt",
    .probe = usbtouch_probe,
    .disconnect	= usbtouch_disconnect,
    .suspend = usbtouch_suspend,
    .resume = usbtouch_resume,
    .reset_resume = usbtouch_reset_resume,
    .id_table = usbtouch_devices,
    .supports_autosuspend = 1
};

static int __init usbtouch_init(void)
{
    return usb_register(&usbtouch_driver);
}

static void __exit usbtouch_cleanup(void)
{
    usb_deregister(&usbtouch_driver);
}

module_init(usbtouch_init);
module_exit(usbtouch_cleanup);

MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL");

MODULE_ALIAS("touchkitusb");
MODULE_ALIAS("itmtouch");
MODULE_ALIAS("mtouchusb");
