#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/input.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/usb.h>
#include <linux/usb/input.h>
#include <linux/input/mt.h>
#include <linux/workqueue.h>

#ifndef err
#define err(f, arg...) printk(KERN_ERR": " f "\n" , ## arg)
#endif

#ifndef dbg
#define dbg(f, arg...) printk(KERN_ERR": " f "\n" , ## arg)
#endif

#ifdef DKMS
#include "include/Device.h"
#else
#include "../core/Device.h"
#endif

/* device specifc data/functions */
struct usbtouch_usb;
struct usbtouch_device_info {
    int min_xc, max_xc;
    int min_yc, max_yc;
    int min_press, max_press;
    int rept_size;

    /*
  * Always service the USB devices irq not just when the input device is
  * open. This is useful when devices have a watchdog which prevents us
  * from periodically polling the device. Leave this unset unless your
  * touchscreen device requires it, as it does consume more of the USB
  * bandwidth.
  */
    bool irq_always;

    void (*process_pkt) (struct usbtouch_usb *usbtouch, unsigned char *pkt, int len);

    /*
  * used to get the packet len. possible return values:
  * > 0: packet len
  * = 0: skip one byte
  * < 0: -return value more bytes needed
  */
    int  (*get_pkt_len) (unsigned char *pkt, int len);

    int  (*read_data)   (struct usbtouch_usb *usbtouch, unsigned char *pkt);
    int  (*alloc)       (struct usbtouch_usb *usbtouch);
    int  (*init)        (struct usbtouch_usb *usbtouch);
    void (*exit)	(struct usbtouch_usb *usbtouch);
};

/* a usbtouch device */
typedef struct usbtouch_usb {
    unsigned char *data;
    dma_addr_t data_dma;
    unsigned char *buffer;
    int buf_len;
    struct urb *irq;
    struct usb_interface *interface;
    struct input_dev *input;
    struct usbtouch_device_info *type;
    char name[128];
    char phys[64];
    void *priv;
    struct work_struct data_work;
} _usbtouch_usb, pusbtouch_usb;

enum {
    DEVTYPE_IGNORE = -1,
    DEVTYPE_DPX,
};

static const struct usb_device_id usbtouch_devices[] = {
    {USB_DEVICE(DPX_VENDOR, DPX_PRODUCT), .driver_info = DEVTYPE_DPX},
    //{USB_DEVICE(DPX_VENDOR, 0x0003), .driver_info = DEVTYPE_DPX}, // This is Skin Ultra in DFU mode
    {}
};

struct dpx_priv {
    PDPX_DEVICE context;
};

void dpx_set_gain(struct usbtouch_usb *usbtouch);
void dpx_calibration(struct usbtouch_usb *usbtouch, unsigned char type);

