#ifndef DPX_ACQUISITION_H
#define DPX_ACQUISITION_H

#include "Config.h"

#define ZEROS_LENGTH 23
#define DATA_LENGTH 32
#define STRING_LENGTH 32

typedef struct _FRAME_PACKET
{
    unsigned short Index;
    unsigned char Status;
    unsigned char Size;
    unsigned char Gain;
    unsigned char TouchPoints;
    unsigned char VersionMajor;
    unsigned char VersionMinor;
    unsigned char AdvancedCalibration;
    unsigned char Zeros[ZEROS_LENGTH];
    unsigned char Data[DATA_LENGTH];
} FRAME_PACKET, *PFRAME_PACKET;

typedef struct _SYNC_PACKET
{
    unsigned char Signature[16]; // DISPLAX-SKIN-V30
    unsigned char VersionMajor;
    unsigned char VersionMinor;
    unsigned char Cols;
    unsigned char Rows;
    unsigned char Status;
    unsigned char Gain;
    unsigned char TouchPoints;
    unsigned char Interlaced;
    unsigned char Zeros[40];
} SYNC_PACKET, *PSYNC_PACKET;

typedef struct _DPX_ACQUISITION_STATE
{
    unsigned char Status;
    unsigned char Gain;
    unsigned char DigitalGain;
    unsigned char Frame;
    unsigned char Interlaced;
    unsigned char FrameRate;
    unsigned char TouchPoints;
    unsigned char VersionMajor;
    unsigned char VersionMinor;
    unsigned char Manufacturer[STRING_LENGTH];
    unsigned char Product[STRING_LENGTH];
    unsigned char Serial[STRING_LENGTH];
    unsigned char CurrentFrame[DPX_FRAME_MAX_SIZE];
    int Size;
    int DriverVersion;
    int FrameCounter;
    int FrameCounterThreshold;
    int AdvancedCalibration;
} DPX_ACQUISITION_STATE, *PDPX_ACQUISITION_STATE;

typedef struct _DPX_ACQUISITION_DATA
{
    unsigned char Synced;
    unsigned char HasReferenceFrame;
    unsigned short TransmittedBytes;
    unsigned int Frames;
    unsigned char Buffer[DPX_FRAME_MAX_SIZE];
    unsigned char ReferenceFrame[DPX_FRAME_MAX_SIZE];
    unsigned char Carry[24];
    unsigned char CarryCount;
} DPX_ACQUISITION_DATA, *PDPX_ACQUISITION_DATA;

int DPXDeviceFillFrameV1(PDPX_DEVICE device, void* buffer, int length);
int DPXDeviceFillFrameV3(PDPX_DEVICE device, void* buffer, int length);

#endif // DPX_ACQUISITION_H
