#ifndef CONTEXT_H
#define CONTEXT_H

#include "Config.h"
#include "Device.h"

#ifdef CONTROLPANEL
class MainWindow;
#endif

#ifdef LINUX
#include <libusb-1.0/libusb.h>
#endif

#define DPX_AUTO_ADJUSTING 1 << 0
#define DPX_AUTO_ADJUSTING_FAILED 1 << 1
#define DPX_AUTO_ADJUSTING_DONE 1 << 2
#define DPX_AUTO_ADJUSTING_IN_PROGRESS 1 << 3

enum {
    DPX_REMOTE_CONNECT,
    DPX_REMOTE_DISCONNECT,
    DPX_REMOTE_ACK,
    DPX_REMOTE_NACK,
    DPX_REMOTE_REQ_UPDATE,
    DPX_REMOTE_REQ_AUTH,
    DPX_REMOTE_AUTH_PIN,
    DPX_REMOTE_AUTH_ACK,
    DPX_REMOTE_AUTH_NACK,
    DPX_REMOTE_REQ_MANUAL_SETUP,
    DPX_REMOTE_SET_SENSITIVITY,
    DPX_REMOTE_SET_NOISE_REDUCTION,
    DPX_REMOTE_SET_ENABLED,
    DPX_REMOTE_DEVICE_STATE,
    DPX_REMOTE_INCOMPATIBLE
};

enum {
    DPX_REMOTE_CALIBRATION_GET_STATUS,
    DPX_REMOTE_CALIBRATION_GET_TOUCH,
    DPX_REMOTE_CALIBRATION_STATUS,
    DPX_REMOTE_CALIBRATION_SET_CALIBRATION_DATA,
    DPX_REMOTE_CALIBRATION_DONE
};

typedef struct _DPX_REMOTE_MSG
{
    DPX_DEVICE_STATE state;
    int msg;
    int content;
} DPX_REMOTE_MSG, *PDPX_REMOTE_MSG;

typedef struct _DPX_REMOTE_CALIBRATION_MSG
{
    int msg;
    int field1;
    int field2;
    int field3;
    int field4;
    int field5;
    int field6;
} DPX_REMOTE_CALIBRATION_MSG, *PDPX_REMOTE_CALIBRATION_MSG;

class Settings;

typedef void (*DPXContextDeviceCallback)(PDPX_DEVICE device, void*);

struct _DPX_CONTEXT
{
    int Count;
    DPX_DEVICE Devices[DPX_MAX_DEVICES];
    
    // callbacks
    void * Data;
    DPXContextDeviceCallback DeviceAdded;
    DPXContextDeviceCallback DeviceRemoved;
};

struct _DPX_SHARED_CONTEXT
{
    PDPX_CONTEXT context;
    PDPX_DEVICE device;
};

PDPX_DEVICE DPXContextGetDeviceWithId(PDPX_CONTEXT Context, const char* Id);
PDPX_DEVICE DPXContextAllocDevice(PDPX_CONTEXT Context, const char* Id, const char* Serial, int Pid);

#ifdef WINDOWS
DWORD WINAPI DPXDeviceRunLoop(PDPX_DEVICE device);
#else
int DPXDeviceRunLoop(PDPX_DEVICE device);
#endif

void DPXContextUpdateDevicesImpl(PDPX_CONTEXT context);

#endif // CONTEXT_H
