#ifndef DPX_PROCESSING_H
#define DPX_PROCESSING_H

#ifdef LINUX_DRIVER
#include <linux/string.h>
#endif
#include "Config.h"
#include "Block.h"

typedef struct _DPX_PROCESSING_STATE
{
    unsigned char Frame[DPX_FRAME_MAX_ROWS][DPX_FRAME_MAX_ROWS];
    int NoiseReduction;
    int PaddingBytes;
} DPX_PROCESSING_STATE, *PDPX_PROCESSING_STATE;

typedef struct _DPX_SEGMENT
{
    int Area;
    int Blocks;
    int Matches;
    int Valid;
    long Average;
} DPX_SEGMENT, *PDPX_SEGMENT;

typedef struct _DPX_PROCESSING_DATA
{
    DPX_BLOCK Blocks[DPX_FRAME_MAX_ROWS][DPX_FRAME_MAX_COLS];

    int ValidBlockCount;
    PDPX_BLOCK ValidBlocks[DPX_FRAME_MAX_SIZE];

    //int Frame[DPX_FRAME_MAX_ROWS][DPX_FRAME_MAX_ROWS];
    int FramePassY[DPX_FRAME_MAX_ROWS][DPX_FRAME_MAX_COLS];
    int FrameBlurKernel[16];    

    int Segment[DPX_FRAME_MAX_ROWS][DPX_FRAME_MAX_ROWS];

    DPX_SEGMENT Segments[DPX_FRAME_MAX_SIZE];

    int CurrentSegment;
    int FillCount;
    int FillQueueX[DPX_FRAME_MAX_SIZE];
    int FillQueueY[DPX_FRAME_MAX_SIZE];

    int FrameBlurWidth;
} DPX_PROCESSING_DATA, *PDPX_PROCESSING_DATA;

void DPXProcessingInitialize(PDPX_DEVICE Device, int noiseReduction);
void DPXProcessingExecute(PDPX_DEVICE Device);
void DPXProcessingGaussianBlur(PDPX_DEVICE context);
void DPXProcessingFindPeaks(PDPX_DEVICE device);
void DPXProcessingSegmentPeaks(PDPX_DEVICE device);
void DPXProcessingSegmentFrame(PDPX_DEVICE device);
void DPXProcessingFloodFillFrameIterative(PDPX_DEVICE device, int segment, int x, int y);

#endif // DPX_PROCESSING_H
