#ifndef DPX_TOUCH_H
#define DPX_TOUCH_H

#include "Config.h"
#include "Point.h"
#include "poly.h"

#define DPX_TOUCH_STATE_INACTIVE        (1 << 0)
#define DPX_TOUCH_STATE_DETECTED        (1 << 1)
#define DPX_TOUCH_STATE_ACTIVE          (1 << 2)

/* Structures */

typedef struct _DPX_VECTOR {
    int X;
    int Y;
    int L;
} DPX_VECTOR, *PDPX_VECTOR;

typedef struct _DPX_CALIBRATION_ZONE
{
    poly Poly;
    long long Matrix[3][3];
    DPX_POINT T0Center;
    DPX_POINT T1Center;
    int valid;
    int unused;
} DPX_CALIBRATION_ZONE, *PDPX_CALIBRATION_ZONE;

typedef struct _DPX_TOUCH_POINT
{
    DPX_POINT Position;
    int Pressure;
} DPX_TOUCH_POINT, *PDPX_TOUCH_POINT;

typedef struct _DPX_TOUCH
{
    int Index;
    int Id;
    int ReportedState;
    int CurrentState;

    int dX, dY, dP, W;

    int FramesCount;
    int FramesDetected;
    int FramesIdle;

    DPX_TOUCH_POINT DetectedPoints[DPX_TOUCH_HISTORY_SIZE];
    DPX_TOUCH_POINT ReportedPoints[DPX_TOUCH_HISTORY_SIZE];

    DPX_POINT Estimation;

    int Current;
    int Last;
    int Count;

    int Confidence;

    int TotalDx;
    int TotalDy;

    DPX_TOUCH_POINT CalibratedPoint;

} DPX_TOUCH, *PDPX_TOUCH;


/* Macros */

#define DPX_TOUCH_FOREACH(T,Device) for(T=(Device)->State.Touches; T!=(Device)->State.Touches+DPX_TOUCH_MAX_COUNT; ++T)

/* API */

PDPX_TOUCH DPXTouchCreate(PDPX_DEVICE Device);
void DPXTouchUpdateDetectedPoint(PDPX_TOUCH Touch, long X, long Y, int Pressure);
void DPXTouchUpdateReportedPoint(PDPX_TOUCH Touch);
void DPXTouchSetState(PDPX_DEVICE Device, PDPX_TOUCH Touch, int state);
void DPXTouchCalibrate(PDPX_DEVICE Device, PDPX_TOUCH Touch);
int DPXDotProduct(PDPX_VECTOR a, PDPX_VECTOR b);
unsigned char DPXIsPointInZoneTriangle(PDPX_CALIBRATION_ZONE zone, int triangle, int a, int b);

#endif // DPX_TOUCH_H
